﻿using Infragistics;
using System; 
using System.Collections.ObjectModel;
using System.Collections.Specialized; 

namespace MultiSurfaceChartApp
{  
    public class DataCollection<T> : ObservableCollection<T>
    {
        public void OnCollectionChanged(NotifyCollectionChangedAction action)
        {
            OnCollectionChanged(new NotifyCollectionChangedEventArgs(action));
        }
    }
     
    public class DataModel : DataCollection<DataPoint3D>
    {
        private static Random random = new Random();

        public DataModel() : this(10, double.NaN)
        {
        }
        public DataModel(int count, double seed)
        {
            if (double.IsNaN(seed))
                seed = random.NextDouble();

            for (int i = 0; i < count; ++i)
            {
                Add(Create(seed));
            }
        }

        public DataPoint3D Create(double seed)
        {
            var v = new DataPoint3D();
            v.X = 100.0 * random.NextDouble();
            v.Y = 100.0 * random.NextDouble();
            v.Z = MathUtil.Noise(0.02 * v.X, 0.02 * v.Y, seed);
            return v;
        }

        public void Update(double seed)
        {
            seed += 0.01;
            foreach (var item in this)
            {
                Update(item, seed);
            }
            OnCollectionChanged(NotifyCollectionChangedAction.Reset);
        }       
        
        public void Update(DataPoint3D v, double seed)
        {
            var z = MathUtil.Noise(0.02 * v.X, 0.02 * v.Y, seed);
            //v.UpdateZ(z);
            v.Z = z; 
        } 
    }

}
