﻿using Infragistics.Controls.Charts;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.Editors;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.IO;
using Microsoft.Win32;
using System.Windows.Media.Imaging;
using System.Windows.Threading;

namespace MultiSurfaceChartApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private MatrixDataContainer _dataCollection;
        public event PropertyChangedEventHandler PropertyChanged;

        public MatrixDataContainer DataCollection
        {
            get { return _dataCollection; }
            private set
            {
                _dataCollection = value;
                OnPropertyChanged(new PropertyChangedEventArgs("DataCollection"));
            }
        }

        public MainWindow()
        {
            InitializeComponent();
            Create3DCube();
            Bind3DCube();

            var updateTimer = new DispatcherTimer { Interval = TimeSpan.FromSeconds(0.5) };
            updateTimer.Tick += updateTimer_Tick;
            updateTimer.Start();
        }


        int use = 0;
        private void updateTimer_Tick(object sender, EventArgs e)
        {
            DataCollection.SetXIndicator(use);
            DataCollection.SetYIndicator(use);

            use++;

            if (use >= 16)
            {
                use = 0;
            }
        }

        private void OnPropertyChanged(PropertyChangedEventArgs ea)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, ea);
            }
        }

        public void Create3DCube()
        {
            try
            {
                this.matrixDataSurfaceChart.XAxis.Title = "RPM";
                this.matrixDataSurfaceChart.YAxis.Title = "TPS";
                this.matrixDataSurfaceChart.ZAxis.Title = "Value";

                zAxis.MinimumValue = 0;
                zAxis.MaximumValue = 25;
                zAxis.Interval = 5;

                this.matrixDataSurfaceChart.RefreshAxes();
            }
            catch
            {

            }
        }

        public void Bind3DCube()
        {
            try
            {
                xAxis.Interval = (Convert.ToDouble(100));
                xAxis.MaximumValue = (Convert.ToDouble(100)) * (16 - 1);

                yAxis.MaximumValue = 100;
                yAxis.Interval = (Convert.ToDouble(100) / 16);
                Random rr = new Random();
                List<DataPoint3D> MatrixData = new List<DataPoint3D>();
                int index = 0;

                for (int x = 0; x < 16; x++)
                {
                    for (int y = 0; y < 16; y++)
                    {
                        DataPoint3D p = new DataPoint3D();

                        p.X = x;
                        p.Y = y;
                        p.Index = index;
                        p.Z = x + 1;
                        MatrixData.Add(p);

                        index++;
                    }
                }

                this.DataCollection = new MatrixDataContainer(MatrixData, 100, 100, true, true, 16);
                this.matrixDataSurfaceChart.ItemsSource = null;
                this.matrixIndicatorXSurfaceChart.ItemsSource = null;
                this.matrixIndicatorYSurfaceChart.ItemsSource = null;
                this.matrixSelectedSurfaceChart.ItemsSource = null;

                this.matrixDataSurfaceChart.ItemsSource = this.DataCollection.DataPoints;
                this.matrixIndicatorXSurfaceChart.ItemsSource = this.DataCollection.IndicatorXPoints;
                this.matrixIndicatorYSurfaceChart.ItemsSource = this.DataCollection.IndicatorYPoints;
                this.matrixSelectedSurfaceChart.ItemsSource = this.DataCollection.SelectedPoints;
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(ex.ToString());
            }
        }     

        public class VisibilityConverter : IValueConverter
        {
            public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
            {
                if (value is bool)
                {
                    return (bool)value ? Visibility.Visible : Visibility.Hidden;
                }

                return Visibility.Collapsed;
            }

            public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
            {
                if (value is Visibility)
                {
                    return ((Visibility)value) == Visibility.Visible;
                }
                return false;
            }
        }

        private void xIndexSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
          
        }

        private void yIndexSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {

        }
    }
}
