﻿using MultiSurfaceChartApp.Models;
using System;
using System.Collections.Generic;

namespace MultiSurfaceChartApp
{
    public class MatrixDataContainer : ObservableModel
    {
        public double xInterval { get; set; }
        public double yInterval { get; set; }

        public List<DataPoint3D> MatrixData { get; set; }

        public DataCollection DataPoints { get; set; }

        public DataCollection IndicatorXPoints { get; set; }
        public DataCollection IndicatorYPoints { get; set; }

        public DataCollection SelectedPoints { get; set; }

        public MatrixDataContainer(List<DataPoint3D> matrixData, double maxX, double maxY, bool isMetric, bool IsTPSMatrix, int matrixSize)
        {
            MatrixData = matrixData;

            DataPoints = new DataCollection();
            IndicatorXPoints = new DataCollection();
            IndicatorYPoints = new DataCollection();
            SelectedPoints = new DataCollection();

            CreateData(matrixData, maxX, maxY, isMetric, IsTPSMatrix, matrixSize);

            SetXIndicator(0);
            SetYIndicator(0);
        }

        public void CreateData(List<DataPoint3D> matrixData, double maxX, double maxY, bool isMetric, bool IsTPSMatrix, int matrixSize)
        {
            xInterval = maxX;

            if (IsTPSMatrix)
            {
                yInterval = Math.Round(Convert.ToDouble(Convert.ToDouble(100) / Convert.ToDouble(matrixSize)),2);
            }
            else
            {
                yInterval = Math.Round(Convert.ToDouble(maxY / Convert.ToDouble(matrixSize)),2);
            }

            foreach (DataPoint3D dp in matrixData)
            {
                DataPoint3D dpp = new DataPoint3D();
                dpp.X = (Math.Abs(dp.X) * xInterval);

                if (IsTPSMatrix)
                {
                    dpp.Y = Convert.ToDouble(100 - (yInterval * Convert.ToDouble(dp.Y)));
                }
                else
                {
                    dpp.Y = Convert.ToDouble((Convert.ToDouble((Convert.ToDouble(maxY) / 10)) - (dp.Y * Convert.ToDouble(Convert.ToDouble((Convert.ToDouble(maxY) / 10)) / (Convert.ToDouble(matrixSize) - 1)))));
                }

                dpp.Z = dp.Z;
                dpp.Index = dp.Index;

                DataPoints.Add(dpp);
            }
        }

        public void Update(int index, double val)
        {
            DataPoints.Update(index, val);
        }

        public void Update(List<DataPoint3D> newPoints)
        {
            DataPoints.Update(newPoints);
        }

        public void SetXIndicator(int xIndex)
        {
            this.IndicatorXPoints.BeginUpdate();
            try
            {

                this.IndicatorXPoints.Clear();

                double findXStart = ((double)xIndex * xInterval); ;
                double findXEnd = ((double)xIndex * xInterval) + xInterval;

                List<DataPoint3D> availableXPoints = DataPoints.ListXIndexPoints(findXStart, findXEnd);
                availableXPoints.ForEach(c => c.Z = (c.Z + 0.00001));
             

                this.IndicatorXPoints.SetPoints(availableXPoints);

            }
            catch
            {
            }
            this.IndicatorXPoints.EndUpdate();
        }

        public void SetYIndicator(int yIndex)
        {
            this.IndicatorYPoints.BeginUpdate();
            try
            {
                this.IndicatorYPoints.Clear();

                double findYStart = ((double)yIndex * yInterval);
                double findYEnd = ((double)yIndex * yInterval) + yInterval;

                List<DataPoint3D> availableYPoints = DataPoints.ListYIndexPoints(findYStart, findYEnd);
                availableYPoints.ForEach(c => c.Z = (c.Z + 0.00001));

                this.IndicatorYPoints.SetPoints(availableYPoints);

            }
            catch (Exception ex)
            {
                string s = ex.ToString();
            }
            this.IndicatorYPoints.EndUpdate();
        }

        public void SetSelected(List<int> indexes)
        {
            try
            {
                this.SelectedPoints.Clear();

                List<DataPoint3D> availableYPoints = DataPoints.ListSelectedPoints(indexes);
                availableYPoints.ForEach(c => c.Z = (c.Z + 0.00002));

                this.SelectedPoints.SetPoints(availableYPoints);

            }
            catch (Exception ex)
            {
                string s = ex.ToString();
            }
        }
    }
}
